// Power Render/WinAmp Plugin Example
// Copyright (C) 1997, Chris Egerter
// Feel free to use this as a framework for other plugins
// Originally modified from the vis_test by Justin Frankel


/*

To recompile with MSVC:

Make sure the DirectX, PR, and Glide include directories are set up in 
the Tools/Options/Directories settings:
ie

d:\dxsdk\sdk\inc
d:\dxsdk\sdk\samples\misc
d:\pr\winclude\msglide
d:\glide24\glide\src\sst1\include
d:\glide24\glide\src\swlibs\texus\lib

Make sure you have the library directories set up as well:

d:\dxsdk\sdk\lib
d:\pr\winlib\msglide
d:\glide24\glide\lib\win32

Select the project and go to Project/Settings/Link,
add the following libraries to the Object/Library Modules listing:

pr.lib fxguir.lib glide2x.lib texus.lib winmm.lib 

FINALLY make sure you are using 1 byte aligned structures.
You can select this from the Project/Settings/ C/C++ under the
Code Generation category.


Known bugs:
As far as I can tell, there is no way to know where the dll is
located, so you can't load the data files from anywhere.
This means you need your mp3 files in the same directory as the
plugin files.

*/


#include <windows.h>
#include "vis.h"

#include <math.h>
#include <stdlib.h>
#include <pr.h>
#include <pr3dfx.h>
#include <glide.h>
#include <prgui.h>
#include <winutil.h>
#include "prbuild.h"


PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *entity;                     /* A single entity of the shape */
PR_OBJECT *back_shape;                 /* The shape loaded */
PR_ENTITY *backentity;                 /* A single entity of the shape */
PR_LIGHTLIST userlights;               /* Holds a list of lights */
PR_DWORD  device=0;                    /* Display device */
PR_UDWORD BLACK;                       /* Color index/value for black */
PR_DWORD  vwidth, vheight;

PR_DWORD backtex;
PR_REAL  squiggle_scale = 25.0f;

/* Each section of the squiggle object has 4 vertices.
   The original position must be stored so it can be added to during
   the realtime spectrum display */
PR_POINT vposition[4];  


void InitializePR (void);

/* dummy routines that are normally defined in winutil but we
   don't need them */
void msetxy (int x, int y) { }
char kbdon[256];
PR_MouseCallback_data mouse;
void msetbounds (int x0, int y0, int x1, int y1) {}
void noclick (void) {}
void UpdateMessages (void) {}
int PR_kbhit (void) { return 0; }
char PR_getch (void) { return 0; }



char szAppName[] = "SimpleVis"; // Our window class, etc

// configuration declarations
int config_x=50, config_y=50;	// screen X position and Y position, repsectively
void config_read(struct winampVisModule *this_mod);		// reads the configuration
void config_write(struct winampVisModule *this_mod);	// writes the configuration
void config_getinifn(struct winampVisModule *this_mod, char *ini_file); // makes the .ini file filename


// returns a winampVisModule when requested. Used in hdr, below
winampVisModule *getModule(int which);

// "member" functions
void config(struct winampVisModule *this_mod); // configuration dialog
int init(struct winampVisModule *this_mod);	   // initialization for module
int render1(struct winampVisModule *this_mod);  // rendering for module 1
void quit(struct winampVisModule *this_mod);   // deinitialization for module

// our window procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
HWND hMainWnd; // main window handle

// Module header, includes version, description, and address of the module retriever function
winampVisHeader hdr = { VIS_HDRVER, "Power Render 3Dfx Squiggle", getModule };

// first module (oscilliscope)
winampVisModule mod1 =
{
	"3Dfx Squiggle",
	NULL,	// hwndParent
	NULL,	// hDllInstance
	0,		// sRate
	0,		// nCh
	25,		// latencyMS
	25,		// delayMS
	2,		// spectrumNch
	0,		// waveformNch
	{ 0, },	// spectrumData
	{ 0, },	// waveformData
	config,
	init,
	render1, 
	quit
};


// this is the only exported symbol. returns our main header.
// if you are compiling C++, the extern "C" { is necessary, so we just #ifdef it
#ifdef __cplusplus
extern "C" {
#endif
__declspec( dllexport ) winampVisHeader *winampVisGetHeader()
{
	return &hdr;
}
#ifdef __cplusplus
}
#endif

// getmodule routine from the main header. Returns NULL if an invalid module was requested,
winampVisModule *getModule(int which)
{
	switch (which)
	{
		case 0: return &mod1;
		default:return NULL;
	}
}

// configuration. Passed this_mod, as a "this" parameter. Allows you to make one configuration
// function that shares code for all your modules (you don't HAVE to use it though, you can make
// config1(), config2(), etc...)
void config(struct winampVisModule *this_mod)
{
	MessageBox(this_mod->hwndParent,"This module is Copyright(C) 1997, Chris Egerter (GooRoo on IRC)\n"
									"Visit http://www.egerter.com for other fun stuff!\n"
									,"Configuration",MB_OK);
}

// initialization. Registers our window class, creates our window, etc. Again, this one works for
// both modules, but you could make init1() and init2()...
// returns 0 on success, 1 on failure.
int init(struct winampVisModule *this_mod)
{
int width = 288; 
int height = 256; 
char path[MAX_PATH];
int len;

	config_read(this_mod);

	GetModuleFileName(this_mod->hDllInstance,path,MAX_PATH);
    PRGUI_InitPath (path);
    PRGUI_GoStartPath ();   

	
	{	// Register our window class
		WNDCLASS wc;
		memset(&wc,0,sizeof(wc));
		wc.lpfnWndProc = WndProc;				// our window procedure
		wc.hInstance = this_mod->hDllInstance;	// hInstance of DLL
		wc.lpszClassName = szAppName;			// our window class name
	    wc.hCursor = NULL;	

		if (!RegisterClass(&wc)) 
		{
			MessageBox(this_mod->hwndParent,"Error registering window class","blah",MB_OK);
			return 1;
		}
	}


	hMainWnd = CreateWindowEx(
		WS_EX_TOOLWINDOW|WS_EX_APPWINDOW,	// these exstyles put a nice small frame, 
											// but also a button in the taskbar
		szAppName,							// our window class name
		this_mod->description,				// use description for a window title
		WS_VISIBLE|WS_SYSMENU,				// make the window visible with a close button
		config_x,config_y,					// screen position (read from config)
		width,height,						// width & height of window (need to adjust client area later)
		this_mod->hwndParent,				// parent window (winamp main window)
		NULL,								// no menu
		this_mod->hDllInstance,				// hInstance of DLL
		0); // no window creation data

	if (!hMainWnd) 
	{
		MessageBox(this_mod->hwndParent,"Error creating window","blah",MB_OK);
		return 1;
	}


	SetWindowLong(hMainWnd,GWL_USERDATA,(LONG)this_mod); // set our user data to a "this" pointer

	{	// adjust size of window to make the client area exactly width x height
		RECT r;
		GetClientRect(hMainWnd,&r);
		SetWindowPos(hMainWnd,0,0,0,width*2-r.right,height*2-r.bottom,SWP_NOMOVE|SWP_NOZORDER);
	}

	// show the window
	ShowWindow(hMainWnd,SW_SHOWNORMAL);

    InitializePR ();
	return 0;
}

// render function for oscilliscope. Returns 0 if successful, 1 if visualization should end.
int render1(struct winampVisModule *this_mod)
{
int x;
int hgt;
int i;
PR_VERTEX *v;
int lx, ly, lx2, ly2;
float circleval;


  v = object_shape->segment_list->vertex_list;

/* Modify the squiggle object */
  for (x = 0; x < 144; x ++)
    {
     hgt = ((PR_REAL)(this_mod->spectrumData[0][x])) * squiggle_scale;
     for (i = 0; i < 4; i++)
       {       
		v->y = vposition[i].y + hgt;
		v++;
	   }
	}



     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();                   /* Begin a new frame */

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

//     PR_SetConstantZ (32768);
     //PRGFX_ResizeTexture (0, 0, vwidth-1, vheight-1, backtex, 0);

     PR_TransformLights (&userlights);

     /* Background */
     PR_RotateEntity (backentity, 0.5f,
		                          -0.7f,
                                  -3.1f);   /* Rotate the entity */
     PR_TransformEntity (backentity);      /* Transform the entity */
     PR_RenderEntity (backentity);         /* Render the entity */

     /* Squiggly */
     PR_RotateEntity (entity, 0.01f,
                              1.5f,
                              0.0f);  /* Rotate the entity */
     PR_TransformEntity (entity);      /* Transform the entity */
     PR_RenderEntity (entity);         /* Render the entity */


     PR_RenderFrame ();                /* Draw the scene */

     PR_SetConstantZ (1);
     lx = (vwidth >> 1) + 10;
     ly = vheight >> 1;
     for (x = 0; x < 144; x++)
       {
		circleval = RADIANS((float)x * 10.0f); 
        PRGFX_SetColor (PRGFX_MakeColor (sin(circleval)*31+31, cos(circleval/2)*31+31, sin(circleval/2)*31+31));

		lx2 = (vwidth >> 1) + (cos(circleval) * ((float)this_mod->spectrumData[0][x]+10+x));
        ly2 = (vheight >> 1) + (sin(circleval) * ((float)this_mod->spectrumData[0][x]+10+x));
		PRGFX_Line (lx, ly, lx2, ly2);
        lx = lx2;
        ly = ly2;
	   }

     PR_Flip (1);                      /* Show the background page */


	return 0;
}


// cleanup (opposite of init()). Destroys the window, unregisters the window class
void quit(struct winampVisModule *this_mod)
{
  config_write(this_mod);		// write configuration

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (entity);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();
  PR_Shutdown3Dfx ();


  DestroyWindow(hMainWnd); // delete our window
  UnregisterClass(szAppName,this_mod->hDllInstance); // unregister window class
}


// window procedure for our window
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_CREATE:		return 0;
		case WM_ERASEBKGND: return 0;
		case WM_PAINT:
			{ // update from doublebuffer
				PAINTSTRUCT ps;
				RECT r;
				HDC hdc = BeginPaint(hwnd,&ps);
				GetClientRect(hwnd,&r);
				EndPaint(hwnd,&ps);
			}
		return 0;
		case WM_DESTROY: PostQuitMessage(0); return 0;
		case WM_KEYDOWN: // pass keyboard messages to main winamp window (for processing)
		  switch( wParam )
 		    {
		     case VK_ESCAPE:
		     PostMessage(hwnd, WM_CLOSE, 0, 0);
			 break;
		    }
			break;

		case WM_KEYUP:
			{	// get this_mod from our window's user data
				winampVisModule *this_mod = (winampVisModule *) GetWindowLong(hwnd,GWL_USERDATA);
				PostMessage(this_mod->hwndParent,message,wParam,lParam);
			}
		return 0;
		case WM_MOVE:
			{	// get config_x and config_y for configuration
				RECT r;
				GetWindowRect(hMainWnd,&r);
				config_x = r.left;
				config_y = r.top;
			}
		return 0;
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}


void config_getinifn(struct winampVisModule *this_mod, char *ini_file)
{	// makes a .ini file in the winamp directory named "plugin.ini"
	char *p;

	GetModuleFileName(this_mod->hDllInstance,ini_file,MAX_PATH);
	p=ini_file+strlen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	strcat(ini_file,"plugin.ini");
}


void config_read(struct winampVisModule *this_mod)
{
	char ini_file[MAX_PATH];
	config_getinifn(this_mod,ini_file);
	config_x = GetPrivateProfileInt(this_mod->description,"Screen_x",config_x,ini_file);
	config_y = GetPrivateProfileInt(this_mod->description,"Screen_y",config_y,ini_file);
}

void config_write(struct winampVisModule *this_mod)
{
	char string[32];
	char ini_file[MAX_PATH];

	config_getinifn(this_mod,ini_file);

	wsprintf(string,"%d",config_x);
	WritePrivateProfileString(this_mod->description,"Screen_x",string,ini_file);
	wsprintf(string,"%d",config_y);
	WritePrivateProfileString(this_mod->description,"Screen_y",string,ini_file);
}



void InitializeDevices (void)
{
  if ((device == DEVICE_3DFX) || (device == DEVICE_ANY))
    device = PR_Detect3Dfx ();

  if (device == DEVICE_3DFX)
    {
     PR_Initialize3Dfx ();
    }
}





void InitializePR (void)
{
PR_DWORD i;


  device = DEVICE_3DFX;

  PR_Initialize (3000);                /* Maximum 3000 triangles per frame */
  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */

  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  if (PR_SetMode (vwidth, vheight, 60) != MODE_SUCCESS)
    PR_FatalError ("Cannot set video mode", NULL);

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  PRGUI_GoStartPath ();   

  /* Build the object */
  object_shape = PR_LoadPRO ("squiggle.pro", LOAD_NORMAL);
  if (object_shape == NULL)
     PR_FatalError ("Cannot find the squiggle.pro file", NULL);
  back_shape = PR_LoadPRO ("walls.pro", LOAD_NORMAL);
  if (back_shape == NULL)
     PR_FatalError ("Cannot find the walls.pro file", NULL);

  for (i = 0; i < 4; i++)
    {
     vposition[i].x = object_shape->segment_list->vertex_list[i].x;
     vposition[i].y = object_shape->segment_list->vertex_list[i].y;
     vposition[i].z = object_shape->segment_list->vertex_list[i].z;
    }
 
  /* Create the main entity */
  entity = PR_CreateEntity (object_shape, "squiggly");
  PR_ScaleEntityAbs (entity, 1.0f, 1.0f, 2.0f);

  backentity = PR_CreateEntity (back_shape, "background");
  
  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, object_shape->bbox.radius / 2, object_shape->bbox.radius * 2);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, 400);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);
}

